/*
 * #########################################################################
 * ##       Copyright © Meta Integration Technology, Inc. 1997-2018       ##
 * ##                        All Rights Reserved.                         ##
 * #########################################################################
 * ##                  http://www.metaintegration.com                     ##
 * #########################################################################
 */

// constants
var BROWSE = "browse";
var DOWNLOAD = "download";
 
var args = WScript.arguments;
var executionMode = "";
var paramsFilePath  = "";
var loginUser     = "";
var loginPassword = "";

if( args.length>=1 ) {
	paramsFilePath = WScript.arguments(0);
}
if( args.length>=2 ) {
    executionMode = WScript.arguments(1);
}

var winNetwork = new ActiveXObject("WScript.Network");
var winShell = WScript.CreateObject("WScript.Shell");
var application;
var version;
var baseKey;
var repositoryServer;
var displayFormat;
var activeDirPath;

// Print the current username
WScript.Echo("Running script as user " + winNetwork.UserName + " in domain " + winNetwork.UserDomain);

// Parse the parameter file
try {
	var paramsDoc = new ActiveXObject("Microsoft.XMLDOM");
	paramsDoc.async = false;
	paramsDoc.load(paramsFilePath);
	
	var paramsRoot = paramsDoc.documentElement;
	var paramsRootAttributes = paramsRoot.attributes;
	this.loginUser     = paramsRootAttributes.getNamedItem("loginUser").value;
	this.loginPassword = paramsRootAttributes.getNamedItem("loginPassword").value;
} catch (err) {
    WScript.Echo("Could not read the parameter file. " + err.message);
	WScript.Quit(1);
}

if (executionMode.toLowerCase() == BROWSE) {
	connect();
	if (args.length>=3) {
		var xmlDoc = null;
		browseRepository(WScript.arguments(2));
	}
	disconnect();
} else if (executionMode.toLowerCase() == DOWNLOAD) {
	connect();
	if (args.length>=3) {
		var success = download(WScript.arguments(2));
		if (!success) {
			disconnect();
			WScript.Quit(1);
		}
	}
	disconnect();
}

function connect()
{
	// Start ERStudio
	var comClass = "ERStudio.Application"
	WScript.Echo("Calling COM api " + comClass);
	try {
		this.application = new ActiveXObject(comClass);
	} catch (err) {
		WScript.Echo("Could not connect to ERStudio. " + err.message);
		WScript.Quit(1);
	}
	application.EnableScreenUpdateEx(false,true);
	application.HideWindow();
	
	version = application.ERSFullVersion;
	baseKey = application.GetRegistryKey();
	repositoryServer = readRegistryKey(baseKey + "\\Repository\\RepoServer Home Dir");
	displayFormat = readRegistryKey(baseKey + "\\Repository\\DiagramNameDisplayFormat");
	if (displayFormat==null || displayFormat=="" || displayFormat==0) {
		// Change the settings to force proper display of Diagram names (necessary for building tree structure of projects)
		// Menu Repository / Options... / General / Diagram Name Display Format
		// DiagramNameDisplayFormat=0 => "Diagram Name (Filename.dm1)"
		// DiagramNameDisplayFormat=1 => "Filename.dm1"
		writeRegistryKey(baseKey + "\\Repository\\DiagramNameDisplayFormat", 1, "REG_DWORD");
	}
	activeDirPath = application.RepoActiveDirectory;
	
	// Connect to the repository
	var loginResult = this.application.RepoLogin(loginUser,loginPassword);
	if (loginResult==1) {
		WScript.Echo("Successfully logged in as user: " + loginUser);
	} else {
		var errorCode = this.application.GetLastErrorCode(); // VT_I4
		var errorMessage = this.application.GetLastErrorString();
		WScript.Echo("Could not connect to the repository: " + errorCode + " (" + errorMessage + ")");
		WScript.Quit(1);
	}
}

function browseRepository(outputFilePath)
{
	// create an XML document object
	try {
		xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
	} catch (err) {
		WScript.Echo("Can not create XML document. " + err.message);
		WScript.Quit(1);
	}
	xmlDoc.async = false;
	var rootNode = xmlDoc.createElement("ERStudio");
	xmlDoc.appendChild(rootNode);

	appendPropertyNode(rootNode, "Version", version);
	appendPropertyNode(rootNode, "RegistryKey", baseKey);
	appendPropertyNode(rootNode, "RepositoryServer", repositoryServer);
	appendPropertyNode(rootNode, "DisplayFormat", displayFormat);
	appendPropertyNode(rootNode, "ActiveDirectory", activeDirPath);

	// Browsing diagrams
	var diagrams = this.application.RepoDiagrams();
	if (diagrams!=null) {
		browseDiagrams(diagrams, rootNode);
	}

	// Browsing projects
	var projects = this.application.RepoProjects();
	if (projects!=null) {
		browseProjects(projects, rootNode)
	}

	var objIntro = xmlDoc.createProcessingInstruction("xml","version='1.0' encoding='UTF-8'");
	xmlDoc.insertBefore(objIntro, xmlDoc.childNodes(0));
	xmlDoc.save(outputFilePath);
}

function disconnect()
{
	this.application.RepoLogout();
	this.application.Quit();
}

function browseDiagrams(diagrams, parentNode)
{
	//var count = diagrams.Count;
	var objEnum = new Enumerator(diagrams);
	for(; !objEnum.atEnd(); objEnum.moveNext()) {
		var diagram = objEnum.item();
		if (diagram!=null) {
			browseDiagram(diagram, parentNode);
		}
	}
}

function browseDiagram(diagram, parentNode)
{
	var name = diagram.StringValue;
	appendPropertyNode(parentNode, "Diagram", name)
}

function browseProjects(projects, parentNode)
{
	//var count = projects.Count;
	var objEnum = new Enumerator(projects);
	for(; !objEnum.atEnd(); objEnum.moveNext()) {
		var project = objEnum.item();
		if (project!=null) {
			browseProject(project, parentNode);
		}
	}
}

function browseProject(project, parentNode)
{
	var name = project.Name;
	var desc = project.Description;
	var projectNode = appendPropertyNode(parentNode, "Project", name);
	projectNode.setAttribute("description", desc);
		   
	var projectDiagrams =  project.GetDiagramList();
	if (projectDiagrams!=null) {
		browseDiagrams(projectDiagrams, projectNode);
	}
}

function download(diagramName)
{
	var res = application.RepoGetDiagram(diagramName);
	// Check it exists and return the file
	if (res==1) {
		// success
		return true;
	} else {
		// error
		var errorCode = this.application.GetLastErrorCode(); // VT_I4
		var errorMessage = this.application.GetLastErrorString();
		WScript.Echo("Could not retrieve Diagram: " + errorCode + " (" + errorMessage + ")");
		return false;
	}
}

function readRegistryKey(keyName)
{
	var value = "";
	try {
		value = winShell.RegRead(keyName);
	} catch (err) {
		WScript.Echo(err.message);
	}
	return value;
}

function writeRegistryKey(keyName, value, type)
{
	try {
		winShell.RegWrite(keyName, value, type);
	} catch (err) {
		WScript.Echo(err.message);
	}
}

function appendPropertyNode(parentNode, name, value)
{
	WScript.Echo(name + "=" + value);
	var childNode = xmlDoc.createElement(name);
	parentNode.appendChild(childNode);
	childNode.setAttribute("value", value);
	return childNode;
}
