/*
 * #########################################################################
 * ##       Copyright © Meta Integration Technology, Inc. 1997-2018       ##
 * ##                        All Rights Reserved.                         ##
 * #########################################################################
 * ##                  http://www.metaintegration.com                     ##
 * #########################################################################
 */

// Returns the argument string converted to uppercase.
function Upper(s) {
	return s.toUpperCase();
}

//Returns the last Arg2 characters of the string Arg1. 
//Arg1 and the result are of type string. 
//Arg2 is of type real, but only the integer part is used.
//If Arg2 is negative, then the Arg2 number of characters will be removed from the left-hand side of the Arg1 string.
function Right(s, count) {
	if (count>0) {
		return s.substring(s.length()-count);
	} else {
		return s.substring(-count);
	}
}

// Returns the length of the string argument.
function Len(s) {
	return s.length();
}

// Returns the first Arg2 characters of the string Arg1. 
// Arg1 and the result are of type string. 
// Arg 2 is of type real, but only the integer part is used.
// If Arg2 is negative, then the Arg2 number of characters will be removed from the right-hand side of the Arg1 string. 
function Left(s,count) {
	if (count>0) { 
		return s.substring(0,count); 
	} else { 
		return s.substring(0, s.length()+count); 
	}
};

// Returns the argument string converted to lowercase.
function Lower(s) {
	return s.toLowerCase();
}

// Returns the substring of Arg1 starting at index Arg2 with a length of Arg3 characters.
// Note that first character position is identified as 1 (like VisualBasic), not as zero (like java)
// Arg1 and the result are of type string. Arg2 and Arg3 are of type real, but only the integer part is used. 
// If Arg2 > Len(Arg1), an empty string is returned. 
// Else, if Arg2+Arg3 > Len(Arg1), Arg3 is adjusted to 1+Len(Arg1)-Arg2. 
// If either of Arg2 or Arg3 is negative or if Arg2 is zero, an error is returned.
function Mid(s, start, length) {
	if (start>s.length()) {
		return "";
	}
	else if (start+length>s.length()) {
		length = 1+s.length()-start;
		return s.substring(start-1, start-1+length);
	}
	else if (start<=0 || length<0) {
		return "";
	}
	else {
		return s.substring(start-1, start-1+length);
	}
}

// Returns the substring of Arg1 starting at index Arg2 with a length of Arg3 characters. 
// Note that first character position is identified as 1 (like VisualBasic), not as zero (like java)
// If Arg2 > Len(Arg1), an empty string is returned. 
// Else, if Arg2+Arg3 > Len(Arg1), Arg3 is adjusted to 1+Len(Arg1)-Arg2. 
// If either of Arg2 or Arg3 is negative or if Arg2 is zero, an error is returned.
function Substring(s, start, length) {
	if (start>s.length()) {
		return "";
	}
	else if (start+length>s.length()) {
		length = 1+s.length()-start;
		return s.substring(start-1, start-1+length);
	}
	else if (start<=0 || length<0) {
		return "";
	}
	else {
		return s.substring(start-1, start-1+length);
	}
}

// Returns the position of the first argument string in the second argument string.
// An optional third argument indicates which match to return. 
// If no third argument is given then the position of the first match is returned. 
// If no match is found, 0 is returned. 
// The search is case-sensitive. 
// If Arg1 is the empty string, 0 is returned.
function Find(pattern, s, matchCount) {
	//debugger;
	if (pattern.length==0) {
		return 0;
	}
	else if (matchCount === undefined) {
		// return the first match
		var index = s.indexOf(pattern);
		return index+1;
	}
	else {
		var start = 0;
		var count = 1;
		var index = -1;
		do {
			index = s.indexOf(pattern, start);
			if (index>=0) {
				// found a match
				if (count==matchCount) {
					// found the correct match
					return index+1;
				} else {
					// not the correct match yet, continue iterating
					count++;
					start = index+1;
				}
			} else {
				// no more matches
				return 0;
			}
		} while (true);
	}
}

// Removes whitespace from the beginning and end of a string.
function Trim(s) {
	return s.trim();
}

// Replaces all occurrences of Arg2 in Arg1 with Arg3. The search is case sensitive.
function Substitute(s, pattern, replacement) {
	var regexp = new RegExp('/'+pattern+'/g');
	return s.replace(regexp, replacement);
}

// Replaces a substring according to a regular expression. 
// Search for the Arg2 regular expression in Arg1 and replace it with Arg3.
// Arg4 specifies the options for the replacement:
// "g" specifies that if Arg2 matches more than once then all matches should be substituted.
// "i" specifies that the comparison should be case insensitive. 
// "s", for single-line mode, specifies that the dot (.) matches every character (instead of every character except new line).
// Some characters, like for instance the backslash character "\", need to be escaped to work when using calculated columns.
function RXReplace(s, exp, replacement, options) {
	var regexp = new RegExp('/'+exp+'/'+options);
	return s.replace(regexp, replacement);
}

// Repeats a string a specified number of times.
function Repeat(s, count) {
	// Oracle Nashorn (1.8.0_121) only supports ECMAScript 5.1
	// string.repeat(count) is only available in ECMAScript 6
	var result = "";
	for (var i=0; i<count; i++) {
		result += s;
	}
	return result;
}

// Concatenates (appends) all the arguments into a string. 
// If one argument is given, then the result is the concatenation of all rows. 
// If more than one argument is given, then each row is concatenated.
function Concatenate() {
	var result = "";
	for (var i = 0; i < arguments.length; i++) {
		var argument = arguments[i];
		if (argument!=null) {
			result += argument;
		}
	  }
	return result;
}

// Encodes characters in the string so that the string only contains characters matching the regular expression [.0-9a-zA-Z].
// Column names in TIBCO Spotfire are stored as UTF-16 encoded strings, 
// while variable names in TIBCO Spotfire Statistics Services are built from 8-bit ASCII characters matching [.0-9a-zA-Z]. 
// Therefore the column names that are sent to TIBCO Spotfire Statistics Services must be encoded. 
// This is done automatically when sending data to TIBCO Spotfire Statistics Services via the built-in data functions input handlers. 
// If you need to provide column name input by some other means (e.g., via a document property) 
// you may need to use this function to encode the column names before applying the data function.
function NameEncode(s) {
	return s;
}

// Replaces all substring codes with decoded characters.
// Column names in TIBCO Spotfire are stored as UTF-16 encoded strings, 
// while variable names in TIBCO Spotfire Statistics Services are built from 8-bit ASCII characters matching [.0-9a-zA-Z] or ASCII strings enclosed in grave accents. 
// Therefore the column names that are sent to TIBCO Spotfire Statistics Services must be encoded. 
// Column names received from TIBCO Spotfire Statistics Services are automatically decoded by the built-in data functions output handlers. 
// This function can be used to decode results that have not been automatically decoded.
function NameDecode(s) {
	return s;
} 

// Splits the values in Arg1 into a number of substrings, using Arg2 as the separator. 
// Arg3 specifies which of the values should be returned as a new column. Only one column is returned. 
// Use a negative Arg3 to specify that the split should be done from the end of the string toward the beginning, instead of from the beginning toward the end. 
// For example, if a column called "X" containing the value "a.b" is split into two parts using the separator ".", then Split([X],".",1) will return "a" and Split([X],".",2) will return "b".
function Split(s, separator, index) {
	var parts = s.split(separator);
	if (index>0) {
		return parts[index-1];
	} else {
		return parts[parts.length+index];
	}
}

// RXExtract returns the part of a string (Arg1) that matches a regular expression (Arg2). 
// Arg3 determines which match to use when there are multiple matches.
function RXExtract(s, exp, index) {
	return s;
}
