<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:ot="http://objectTypes.data"
    exclude-result-prefixes="xs" version="2.0">
    
    <xsl:param name="bridgePath"></xsl:param>
    <xsl:param name="bridgeName"></xsl:param>
    
    <xsl:variable name="objectTypeDoc"><xsl:value-of select="$bridgePath"></xsl:value-of>/conf/SASMapping/MIR_ObjectTypes.xml</xsl:variable>
    <xsl:variable name="assocDoc"><xsl:value-of select="$bridgePath"></xsl:value-of>/conf/SASMapping/MIR_Associations.xml</xsl:variable>
    

    <xsl:key name="ot-lookup" match="objectType" use="name"/>
    <xsl:variable name="ot-top" select="document( $objectTypeDoc)"/>
    <xsl:key name="a-lookup" match="association" use="name"/>
    <xsl:variable name="a-top" select="document($assocDoc)"/>
    
    <xsl:key name="schemas" match="//Group[@name='Schemas']/Ent" use="@n"/>

    <xsl:template match="*">
        <xsl:element name="relationshipModels">
            <xsl:choose>
                <xsl:when test="/ProfileOutput/Objects/Ent[@type='Model']">
                    <xsl:for-each select="/ProfileOutput/Objects/Ent[@type='Model']">
                        <xsl:call-template name="StoreModel" />
                    </xsl:for-each>
                </xsl:when>
                <!-- xsl:when test="/ProfileOutput/Objects/Ent[@type='Store Model']">
                    <xsl:for-each select="/ProfileOutput/Objects/Ent[@type='Store Model']">
                        <xsl:call-template name="StoreModel" />
                    </xsl:for-each>
                </xsl:when-->
                <xsl:otherwise>
                    <xsl:apply-templates select="//Ent"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:element>
    </xsl:template>
  
    <xsl:template name="StoreModel">
        <xsl:element name="relationshipModel">
            <xsl:attribute name="version">1</xsl:attribute>
            <xsl:call-template name="resource"/>
            <xsl:element name="relationships">
                <xsl:call-template name="schema"/>
            </xsl:element> 
        </xsl:element>
            <xsl:call-template name="tables"/>
    </xsl:template>
    
    <xsl:template name="schema">
        <xsl:for-each select="./Group[@name='Schemas']">
            <xsl:call-template name="relationship"/>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template name="tables">
        
        <xsl:for-each select="./Group[@name='Tables/Entities']/Ent">
            <xsl:element name="relationshipModel">
                <xsl:attribute name="version">1</xsl:attribute>
                <xsl:element name="resource">
                    <xsl:element name="externalId">
                        <xsl:choose>
                            <xsl:when test="string-length(@id) > 2082">
                                <xsl:value-of select="substring(@id, 1, 2082)"/>
                            </xsl:when>
                            <xsl:when test="string-length(@id) =0">
                                <xsl:value-of select="@n"/>
                                <xsl:value-of select="@mId"/>
                            </xsl:when>                             
                            <xsl:otherwise>   
                                <xsl:value-of select="@id"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:element>
                    <xsl:element name="objectType">
                        <xsl:apply-templates select="$ot-top/*">
                            <xsl:with-param name="mType" select="@mType"/>
                        </xsl:apply-templates>
                    </xsl:element>
                    <xsl:variable name="physicalName"><xsl:for-each select="./Att[@n='Physical Name']"><xsl:value-of select="@v"/></xsl:for-each></xsl:variable>
                    <xsl:choose>
                        <xsl:when test="string-length($physicalName) > 0">
                            <xsl:element name="label">
                                <xsl:value-of select="$physicalName"/>
                            </xsl:element>
                        </xsl:when>
                        <xsl:when test="string-length(@n) > 255">
                            <xsl:element name="label">
                                <xsl:value-of select="substring(@n, 1, 255)"/>
                            </xsl:element>
                        </xsl:when>
                        <!--xsl:when test="string-length(@n) = 0">
                            <xsl:element name="label">unnamed_<xsl:value-of select="@mType"/></xsl:element>
                        </xsl:when-->
                        <xsl:otherwise>
                            <xsl:element name="label">
                                <xsl:value-of select="@n"/>
                            </xsl:element>
                        </xsl:otherwise>
                    </xsl:choose>
                    <xsl:element name="properties">
                    <xsl:choose>
                    <xsl:when test="string-length(@id) > 1024">
                        <xsl:element name="property">
                            <xsl:attribute name="name">externalId</xsl:attribute>
                            <xsl:element name="value">
                                    <xsl:value-of select="substring(@id, 1, 1024)"/>
                            </xsl:element>
                    </xsl:element>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:element name="property">
                            <xsl:attribute name="name">externalId</xsl:attribute>
                            <xsl:element name="value">
                                    <!--xsl:value-of select="@id"/-->
                                <xsl:choose>
                                    <xsl:when test="string-length(@id) > 1024">
                                        <xsl:value-of select="substring(@id, 1, 1024)"/>
                                    </xsl:when>
                                    <xsl:when test="string-length(@id) = 0">
                                        <xsl:value-of select="@n"/>
                                        <xsl:value-of select="@mId"/>
                                    </xsl:when>                                    
                                    <xsl:otherwise>   
                                        <xsl:value-of select="@id"/>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:element>
                        </xsl:element>
                        </xsl:otherwise>
                        </xsl:choose>
                        <xsl:element name="property">
                            <xsl:attribute name="name">source</xsl:attribute>
                            <xsl:element name="value">
                                <xsl:value-of select="$bridgeName"/>
                            </xsl:element>
                        </xsl:element>
                        <xsl:if test="string-length($physicalName) > 0">
                            <xsl:element name="property">
                                <xsl:attribute name="name">description</xsl:attribute>
                                <xsl:element name="value">
                                    <xsl:value-of select="@n"/>
                                </xsl:element>
                            </xsl:element>
                         </xsl:if>
                        
                        <xsl:if test="string-length(@n) > 255">
                            <xsl:element name="property">
                                <xsl:attribute name="name">label</xsl:attribute>
                                <xsl:element name="value">
                                    <xsl:value-of select="@n"/>
                                </xsl:element>
                            </xsl:element>
                        </xsl:if>                        
                        <xsl:for-each select="./Lnk">
                            <xsl:if test=".[@path='PropertyValue']">
                                <xsl:for-each select="./Ent">
                                    <xsl:element name="property">
                                        <xsl:attribute name="name">
                                            <xsl:value-of select="./Att[@n]"/>
                                        </xsl:attribute>
                                        <xsl:element name="value">
                                            <xsl:value-of select="./Att[@v]"/>
                                        </xsl:element>
                                    </xsl:element>
                                </xsl:for-each>
                            </xsl:if>
                            
                        </xsl:for-each>
                    </xsl:element>
                </xsl:element>
                <xsl:element name="relationships">
                    <xsl:variable name="schemaName"><xsl:for-each select="./Att[@n='Owner Schema']"><xsl:value-of select="@v"/></xsl:for-each></xsl:variable>
                    <xsl:variable name="schema" select="key('schemas', $schemaName)"/>
                    <xsl:variable name="schemaId"><xsl:value-of select="$schema/@id"></xsl:value-of></xsl:variable>
                    <xsl:variable name="schemaType"><xsl:value-of select="$schema/@mType"></xsl:value-of></xsl:variable>
                    <xsl:element name="relationship">
                        <xsl:element name="relationshipType">D</xsl:element>
                        <xsl:element name="resource">
                            <xsl:element name="externalId">
                                <xsl:value-of select="$schemaId"/>
                            </xsl:element>
                            <xsl:element name="objectType">
                                <xsl:apply-templates select="$ot-top">
                                    <xsl:with-param name="mType" select="$schemaType"/>
                                </xsl:apply-templates>
                            </xsl:element>
                            <xsl:variable name="physicalName"><xsl:for-each select="./Att[@n='Physical Name']"><xsl:value-of select="@v"/></xsl:for-each></xsl:variable>
                            <xsl:choose>
                                <xsl:when test="string-length($physicalName) > 0">
                                    <xsl:element name="label">
                                        <xsl:value-of select="$physicalName"/>
                                    </xsl:element>
                                </xsl:when>
                                <xsl:when test="string-length(@n) > 255">
                                    <xsl:element name="label">
                                        <xsl:value-of select="substring(@n, 1, 255)"/>
                                    </xsl:element>
                                </xsl:when>
                                <!--xsl:when test="string-length(@n) = 0">
                                    <xsl:element name="label">unnamed_<xsl:value-of select="@mType"/></xsl:element>
                                </xsl:when-->                                
                                <xsl:otherwise>
                                    <xsl:element name="label">
                                        <xsl:value-of select="@n"/>
                                    </xsl:element>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:element>
                        <xsl:element name="properties">
                            <xsl:element name="property">
                                <xsl:attribute name="name">Path</xsl:attribute>
                                <xsl:element name="value">Schema group</xsl:element>
                            </xsl:element>
                            <xsl:element name="property">
                                <xsl:attribute name="name">Type</xsl:attribute>
                                <xsl:element name="value">XSL</xsl:element>
                            </xsl:element>
                            <xsl:variable name="physicalName"><xsl:for-each select="./Att[@n='Physical Name']"><xsl:value-of select="@v"/></xsl:for-each></xsl:variable>
                            <xsl:if test="string-length($physicalName) = 0">
                                <xsl:element name="property">
                                    <xsl:attribute name="name">description</xsl:attribute>
                                    <xsl:element name="value">
                                        <xsl:value-of select="@n"/>
                                    </xsl:element>
                                </xsl:element>
                            </xsl:if>
                            
                            <xsl:if test="string-length(@n) > 255">
                                <xsl:element name="property">
                                    <xsl:attribute name="name">label</xsl:attribute>
                                    <xsl:element name="value">
                                        <xsl:value-of select="@n"/>
                                    </xsl:element>
                                </xsl:element>
                            </xsl:if>
                        </xsl:element>
                    </xsl:element>
                    <xsl:for-each select="./Group">
                        <xsl:call-template name="Group"/>
                    </xsl:for-each>
                    <xsl:for-each select="./Lnk[@path='Feature']">
                        <xsl:call-template name="relationship">
                           <xsl:with-param name="path">Feature</xsl:with-param>
                        </xsl:call-template>
                    </xsl:for-each>
                </xsl:element>
            </xsl:element>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="//Ent">

        <xsl:element name="relationshipModel">
            <xsl:attribute name="version">1</xsl:attribute>
            <xsl:call-template name="resource"/>

        </xsl:element>
    </xsl:template>

    <xsl:template name="resource">
        <xsl:element name="resource">
            <xsl:element name="externalId">
                <xsl:choose>
                    <xsl:when test="string-length(@id) > 2082">
                        <xsl:value-of select="substring(@id, 1, 2082)"/>
                    </xsl:when>
                    <xsl:when test="string-length(@id) = 0">
                        <xsl:value-of select="@n"/>
                        <xsl:value-of select="@mId"/>
                    </xsl:when> 
                    <xsl:otherwise>   
                        <xsl:value-of select="@id"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:element>
            <xsl:element name="objectType">
                <xsl:apply-templates select="$ot-top/*">
                    <xsl:with-param name="mType" select="@mType"/>
                </xsl:apply-templates>
            </xsl:element>
            <xsl:variable name="physicalName"><xsl:for-each select="./Att[@n='Physical Name']"><xsl:value-of select="@v"/></xsl:for-each></xsl:variable>
            <xsl:choose>
                <xsl:when test="string-length($physicalName) > 0">
                    <xsl:element name="label">
                        <xsl:value-of select="$physicalName"/>
                    </xsl:element>
                </xsl:when>
                <xsl:when test="string-length(@n) > 255">
                    <xsl:element name="label">
                        <xsl:value-of select="substring(@n, 1, 255)"/>
                    </xsl:element>
                </xsl:when>
                <!--xsl:when test="string-length(@n) = 0">
                    <xsl:element name="label">unnamed_<xsl:value-of select="@mType"/></xsl:element>
                </xsl:when-->                
                <xsl:otherwise>
                    <xsl:element name="label">
                        <xsl:value-of select="@n"/>
                    </xsl:element>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:element name="properties">
                    <xsl:choose>
                    <xsl:when test="string-length(@id) > 1024">
                        <xsl:element name="property">
                            <xsl:attribute name="name">externalId</xsl:attribute>
                            <xsl:element name="value">
                                    <xsl:value-of select="substring(@id, 1, 1024)"/>
                            </xsl:element>
                    </xsl:element>
                    </xsl:when>
                    <xsl:when test="string-length(@id) = 0">
                        <xsl:element name="property">
                            <xsl:attribute name="name">externalId</xsl:attribute>
                            <xsl:element name="value">
                                    <xsl:value-of select="@n"/>
                                    <xsl:value-of select="@mId"/>
                            </xsl:element>
                    </xsl:element>
                    </xsl:when>                    
                    <xsl:otherwise>
                        <xsl:element name="property">
                            <xsl:attribute name="name">externalId</xsl:attribute>
                            <xsl:element name="value">
                                    <xsl:value-of select="@id"/>
                            </xsl:element>
                        </xsl:element>
                        </xsl:otherwise>
                        </xsl:choose>
                <xsl:element name="property">
                    <xsl:attribute name="name">source</xsl:attribute>
                    <xsl:element name="value">
                        <xsl:value-of select="$bridgeName"/>
                    </xsl:element>
                </xsl:element> 
                <xsl:if test="string-length($physicalName) > 0">
                    <xsl:element name="property">
                        <xsl:attribute name="name">description</xsl:attribute>
                        <xsl:element name="value">
                            <xsl:value-of select="@n"/>
                        </xsl:element>
                    </xsl:element>
                </xsl:if>
                <xsl:if test="string-length(@n) > 255">
                    <xsl:element name="property">
                        <xsl:attribute name="name">label</xsl:attribute>
                        <xsl:element name="value">
                            <xsl:value-of select="@n"/>
                        </xsl:element>
                    </xsl:element>
                </xsl:if>
                    <xsl:for-each select="./Att[@n='Semantics']">
                        <xsl:element name="property">
                            <xsl:attribute name="name">semantics</xsl:attribute>
                            <xsl:element name="value">
                                <xsl:choose>
                                    <xsl:when test="string-length(@v) > 1024">
                                        <xsl:value-of select="substring(@v, 1, 1024)"/>
                                    </xsl:when>
                                    <xsl:otherwise>   
                                        <xsl:value-of select="@v"/>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:element>
                        </xsl:element>
                    </xsl:for-each>
                <xsl:for-each select="./Att[@n='Script']">
                    <xsl:element name="property">
                        <xsl:attribute name="name">semantics</xsl:attribute>
                        <xsl:element name="value">
                            <xsl:choose>
                                <xsl:when test="string-length(@v) > 1024">
                                    <xsl:value-of select="substring(@v, 1, 1024)"/>
                                </xsl:when>
                                <xsl:otherwise>   
                                    <xsl:value-of select="@v"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:element>
                    </xsl:element>
                </xsl:for-each>                
                <xsl:for-each select="./Att[@n='Operation']">
                    <xsl:element name="property">
                        <xsl:attribute name="name">operation</xsl:attribute>
                        <xsl:element name="value">
                            <xsl:choose>
                                <xsl:when test="string-length(@v) > 1024">
                                    <xsl:value-of select="substring(@v, 1, 1024)"/>
                                </xsl:when>
                                <xsl:otherwise>   
                                    <xsl:value-of select="@v"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:element>
                    </xsl:element>
                </xsl:for-each>
                <xsl:for-each select="./Att[@n='Expression']">
                    <xsl:element name="property">
                        <xsl:attribute name="name">expression</xsl:attribute>
                        <xsl:element name="value">
                            <xsl:choose>
                                <xsl:when test="string-length(@v) > 1024">
                                    <xsl:value-of select="substring(@v, 1, 1024)"/>
                                </xsl:when>
                                <xsl:otherwise>   
                                    <xsl:value-of select="@v"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:element>
                    </xsl:element>
                </xsl:for-each>
                 	<xsl:for-each select="./Att[@n='Native Data Type']">
		              <xsl:element name="property">
			             <xsl:attribute name="name">
                             <xsl:value-of select="@n" />
                         </xsl:attribute>
			            <xsl:element name="value">
				             <xsl:value-of select="@v" />
			            </xsl:element>
		             </xsl:element>
	               </xsl:for-each>
                  	<xsl:for-each select="./Att[@n='Size']">
		              <xsl:element name="property">
			             <xsl:attribute name="name">
                             <xsl:value-of select="@n" />
                         </xsl:attribute>
			            <xsl:element name="value">
				             <xsl:value-of select="@v" />
			            </xsl:element>
		             </xsl:element>
	               </xsl:for-each>
                  	<xsl:for-each select="./Att[@n='Scale']">
		              <xsl:element name="property">
			             <xsl:attribute name="name">
                             <xsl:value-of select="@n" />
                         </xsl:attribute>
			            <xsl:element name="value">
				             <xsl:value-of select="@v" />
			            </xsl:element>
		             </xsl:element>
	               </xsl:for-each>                
                </xsl:element>
         </xsl:element>
        <xsl:element name="relationships">
            <xsl:for-each select="./Lnk">
                <!--xsl:if test=".[@path!='OutputTransformationDataSet']">
                    <xsl:if test=".[@path!='Type']">
                        <xsl:if test=".[@path!='PropertyValue']">
                            <xsl:if test=".[@path!='ClassifierMap']">
                            <xsl:call-template name="relationship">
                                <xsl:with-param name="path">
                                    <xsl:value-of select="@path"/>
                                </xsl:with-param>
                            </xsl:call-template>
                        </xsl:if>
                        </xsl:if>
                    </xsl:if>
                </xsl:if-->
                <xsl:variable name="importType">
                    <xsl:apply-templates select="$a-top/*">
                        <xsl:with-param name="path" select="@path"/>
                        <xsl:with-param name="attr">import</xsl:with-param>
                    </xsl:apply-templates>
                </xsl:variable>
                <xsl:if test="$importType='true'">
                    <xsl:call-template name="relationship">
                        <xsl:with-param name="path">
                            <xsl:value-of select="@path"/>
                        </xsl:with-param>
                    </xsl:call-template>
                </xsl:if>
            </xsl:for-each>
            <!--xsl:if test="@mType='Store Model'">
                <xsl:for-each select="./Group">
                    <xsl:call-template name="Group"/>
                </xsl:for-each>
            </xsl:if>
            <xsl:if test="@mType='Class'">
                <xsl:for-each select="./Group">
                    <xsl:call-template name="Group"/>
                </xsl:for-each>
            </xsl:if-->
            <xsl:for-each select="./Group">
                <xsl:call-template name="Group"/>
            </xsl:for-each>
            
            <xsl:for-each select="parent::Lnk">
                <xsl:if test="@path='OutputTransformationDataSet' or @path='OutputConnectionDataSet' ">

                    <xsl:call-template name="relationship_output"> </xsl:call-template>
                </xsl:if>
            </xsl:for-each>
        </xsl:element>
    </xsl:template>

    <xsl:template name="Group">
        <xsl:choose>
            <xsl:when test="@name='Columns/Attributes'">
            <xsl:call-template name="relationship">
                <xsl:with-param name="path">Feature</xsl:with-param>
            </xsl:call-template>
        </xsl:when>
            <xsl:when test="@name='Columns'">
                <xsl:call-template name="relationship">
                    <xsl:with-param name="path">Feature</xsl:with-param>
                </xsl:call-template>
            </xsl:when>
            
        <xsl:otherwise>
            <xsl:call-template name="relationship">
                <xsl:with-param name="path"><xsl:value-of select="@name"/></xsl:with-param>
            </xsl:call-template>
        </xsl:otherwise>
            
        </xsl:choose>
        
    </xsl:template>


    <xsl:template name="relationship">
        <xsl:param name="path"/>

        <xsl:for-each select="./Ent">
            <xsl:element name="relationship">
                <xsl:element name="relationshipType">
                    <xsl:apply-templates select="$a-top/*">
                        <xsl:with-param name="path" select="$path"/>
                    </xsl:apply-templates>
                </xsl:element>
                <xsl:call-template name="rel_resource"/>
                <xsl:element name="properties">
                    <xsl:element name="property">
                        <xsl:attribute name="name">Path</xsl:attribute>
                        <xsl:element name="value"><xsl:value-of select="$path"/></xsl:element>
                    </xsl:element>
                    <xsl:element name="property">
                        <xsl:attribute name="name">Type</xsl:attribute>
                        <xsl:element name="value">XSL</xsl:element>
                    </xsl:element>
                </xsl:element>
            </xsl:element>
        </xsl:for-each>
    </xsl:template>

    <xsl:template name="relationship_output">
        <xsl:param name="path"/>

        <xsl:for-each select="parent::Ent">
            <xsl:element name="relationship">
                <xsl:element name="relationshipType">D</xsl:element>
                <xsl:call-template name="rel_resource"/>
                <xsl:element name="properties">
                    <xsl:element name="property">
                        <xsl:attribute name="name">Path</xsl:attribute>
                        <xsl:element name="value">parent::Ent</xsl:element>
                    </xsl:element>
                    <xsl:element name="property">
                        <xsl:attribute name="name">Type</xsl:attribute>
                        <xsl:element name="value">XSL</xsl:element>
                    </xsl:element>
                </xsl:element>
            </xsl:element>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="objectTypes">
        <xsl:param name="mType"/>
       <xsl:variable name="obtype"> <xsl:value-of select="key('ot-lookup', $mType, $ot-top)/type"/>
       </xsl:variable>
        <xsl:choose>
            <xsl:when test="$obtype=''">-1</xsl:when>
            <xsl:otherwise><xsl:value-of select="$obtype"></xsl:value-of></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="associations">
        <xsl:param name="path"/>
        <xsl:param name="attr">relType</xsl:param>
        <xsl:choose>
            <xsl:when test="$attr='relType'">
                <xsl:variable name="reltype"> <xsl:value-of select="key('a-lookup', $path, $a-top)/relType"/>
                </xsl:variable>
                <xsl:choose>
                    <xsl:when test="$reltype=''">A</xsl:when>
                    <xsl:otherwise><xsl:value-of select="$reltype"></xsl:value-of></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="key('a-lookup', $path, $a-top)/import"/>    
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    
    <xsl:template name="rel_resource">
        <xsl:element name="resource">
            <xsl:element name="externalId">
                <xsl:choose>
                    <xsl:when test="string-length(@id) > 2082">
                        <xsl:value-of select="substring(@id, 1, 2082)"/>
                    </xsl:when>
                    <xsl:when test="string-length(@id) =0">
                        <xsl:value-of select="@n"/>
                        <xsl:value-of select="@mId"/>
                    </xsl:when>                    
                    <xsl:otherwise>   
                        <xsl:value-of select="@id"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:element>
            <xsl:element name="objectType">
                <xsl:apply-templates select="$ot-top">
                    <xsl:with-param name="mType" select="@mType"/>
                </xsl:apply-templates>
            </xsl:element>
            <xsl:variable name="physicalName"><xsl:for-each select="./Att[@n='Physical Name']"><xsl:value-of select="@v"/></xsl:for-each></xsl:variable>
            <xsl:choose>
                <xsl:when test="string-length($physicalName) > 0">
                    <xsl:element name="label">
                        <xsl:value-of select="$physicalName"/>
                    </xsl:element>
                </xsl:when>
                <xsl:when test="string-length(@n) > 255">
                    <xsl:element name="label">
                        <xsl:value-of select="substring(@n, 1, 255)"/>
                    </xsl:element>
                </xsl:when>
                <!--xsl:when test="string-length(@n) = 0">
                    <xsl:element name="label">unnamed_<xsl:value-of select="@mType"/></xsl:element>
                </xsl:when-->                
                <xsl:otherwise>
                    <xsl:element name="label">
                        <xsl:value-of select="@n"/>
                    </xsl:element>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:element>

    </xsl:template>

</xsl:stylesheet>
